import React, { useEffect, useState } from 'react';
import Search from '../../extra/associationSearch';
import { message } from 'antd';
import { ApolloSearchProps } from '../editInterface';
import { antiAssign } from '@/apollo-table/utils/utils';
import s from './index.less';

const ApolloSearch = (props: ApolloSearchProps) => {
    const { onChange, maxCount, isMultiple, request, type, emitTrigger, onEmitChange } = props;
    const selfProps = antiAssign(props, [
        'columnConfig',
        'value',
        'onChange',
        'emitTrigger',
        'onEmitChange',
        'isMultiple',
        'options',
        'request',
    ]);
    const [value, setValue] = useState(props.value);
    useEffect(() => {
        setValue(props.value);
    }, [props.value]);
    const changeValue = (value) => {
        if (maxCount && maxCount < value.length) {
            message.warn(`最多选择${maxCount}个`);
            return;
        }
        setValue(value);
        if (typeof onChange === 'function') {
            onChange(value, value);
        }
        if (!emitTrigger || emitTrigger === 'onChange') {
            emitChange(value, value);
        }
    };
    const emitChange = (value, option) => {
        if (typeof onEmitChange === 'function') {
            onEmitChange(value, option);
        }
    };
    const onBlur = () => {
        emitChange(value, value);
    };
    if (emitTrigger === 'onBlur') {
        selfProps.onBlur = onBlur;
    }
    if (isMultiple) {
        selfProps.mode = 'multiple';
    }
    return (
        <div className={s.searchContainer}>
            <Search
                className={s.select}
                request={(val) => {
                    return request({ name: val, fieldValueName: type });
                }}
                fieldNames={{ value: 'fieldValueValue', label: 'fieldValueName' }}
                {...selfProps}
                value={value}
                onChange={changeValue}
                initDataType="onfocus"
            />
        </div>
    );
};
export default ApolloSearch;
